import yfinance as yf


class StockStat:
    """StockStat class is used to load and store the data from yahoo finance."""
    # 10 best performing stocks of 2021 By yahoo! Finance, Ticker and Company Name.
    # Devon Energy (NYSE:DVN)
    # Marathon Oil (NYSE:MRO)
    # Moderna (NASDAQ:MRNA)
    # Fortinet (NASDAQ:FTNT)
    # Signature Bank (NASDAQ:SBNY)
    # Ford (NYSE:F)
    # Bath & Body Works Inc (NYSE:BBWI)
    # Diamondback Energy (NASDAQ:FANG)
    # Nvidia (NASDAQ:NVDA)
    # Nucor (NYSE:NUE)

    def __init__(self, all_data, ticker, value, graphtype, ax) -> None:
        """Initialize"""
        self.all_data = all_data
        self.__ticker = ticker
        self.__value = value
        self.__graphtype = graphtype
        self.ax = ax

    @property
    def ticker(self):
        """get or set ticker"""
        return self.__ticker

    @ticker.setter
    def ticker(self, ticker: str):
        self.__ticker = ticker

    @property
    def value(self):
        """get or set value"""
        return self.__value

    @value.setter
    def value(self, value: str):
        self.__value = value

    @property
    def graphtype(self):
        """get or set graphtype"""
        return self.__graphtype

    @graphtype.setter
    def graphtype(self, graphtype: str):
        self.__graphtype = graphtype

    def load_data(self):
        self.all_data = yf.download(['DVN', 'MRO', 'MRNA', 'FTNT', 'SBNY', 'F',
                                    'BBWI', 'FANG', 'NVDA', 'NUE'], start='2021-01-01', end='2021-12-31')
        self.all_data.columns = self.all_data.columns.swaplevel(0, 1)
        return self.all_data

    def plotting(self):
        """Set dataframe and Plotting the graph"""
        if len(self.value) == 2:
            return self.all_data[self.ticker][self.value].plot(kind=self.graphtype, xlabel='date', ylabel=f"{self.value[0]} and {self.value[1]}", title=f'{self.value[0]} vs {self.value[1]} chart of {self.ticker}', ax=self.ax, grid=True, logy=False)
        return self.all_data[self.ticker][self.value].plot(kind=self.graphtype, xlabel='date', ylabel=self.value[0], title=f'{self.value[0]} chart of {self.ticker}', ax=self.ax, grid=True, logy=False)
