from threading import Thread
from tkinter import filedialog
from stock_controller import StockController
import tkinter as tk
from tkinter import Toplevel, ttk
import matplotlib
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
matplotlib.use("TkAgg")


class StockUI(tk.Tk):

    def __init__(self, sControl: StockController) -> None:
        super().__init__()
        self.sControl = sControl
        self.frame_panel = tk.LabelFrame(self, text='Select Filters: ')
        self.title('10 best performing stocks of 2021 By yahoo! Finance')
        self.init_components()
        self.show_progress()

    def init_components(self):
        """Initialize components of the application"""
        self.init_label()
        self.init_combobox()
        self.init_button()
        self.init_graph()
        self.compare()
        self.frame_panel.rowconfigure(0, weight=1)
        for num in range(12):
            self.frame_panel.columnconfigure(num, weight=1)
        self.columnconfigure(0, weight=1)
        self.rowconfigure(1, weight=1)

    def compare(self):
        self.ck_var = tk.IntVar()
        self.selectVALUE2 = tk.StringVar()
        self.value_label2 = tk.Label(
            self.frame_panel, text='select Compare: ')
        self.ck_box = tk.Checkbutton(
            self.frame_panel, text='Compare?', variable=self.ck_var, onvalue=1, offvalue=0, command=self.compare_handler)
        self.value2_combobox = ttk.Combobox(
            self.frame_panel, textvariable=self.selectVALUE2)
        self.value2_combobox['values'] = self.sControl.value_for_cbb()
        self.value2_combobox['state'] = 'readonly'

        self.ck_box.grid(row=0, column=4, padx=5, pady=5, sticky="NEWS")

    def compare_handler(self):
        if self.ck_var.get() == 1:
            self.value_label2.grid(
                row=3, column=0, padx=5, pady=5, sticky="NEWS")
            self.value2_combobox.grid(
                row=3, column=1, padx=5, pady=5, sticky="NEWS")

        else:
            self.value_label2.grid_remove()
            self.value2_combobox.grid_remove()

    def init_label(self):
        """Initialize label"""
        self.ticker_label = tk.Label(self.frame_panel, text='select Ticker: ')
        self.value_label = tk.Label(self.frame_panel, text='select Value: ')
        self.graph_label = tk.Label(self.frame_panel, text='select Graph: ')

        self.frame_panel.grid(row=0, column=0, padx=5, pady=5, sticky="NEWS")
        self.ticker_label.grid(row=0, column=0, padx=5, pady=5, sticky="NEWS")
        self.value_label.grid(row=1, column=0, padx=5, pady=5, sticky="NEWS")
        self.graph_label.grid(row=2, column=0, padx=5, pady=5, sticky="NEWS")

    def init_combobox(self):
        """Initialize combobox"""
        self.selectTICKER = tk.StringVar()
        self.selectVALUE = tk.StringVar()
        self.selectGRAPH = tk.StringVar()

        self.ticker_combobox = ttk.Combobox(
            self.frame_panel, textvariable=self.selectTICKER)
        self.value_combobox = ttk.Combobox(
            self.frame_panel, textvariable=self.selectVALUE)
        self.graph_combobox = ttk.Combobox(
            self.frame_panel, textvariable=self.selectGRAPH)

        self.ticker_combobox.grid(
            row=0, column=1, padx=5, pady=5, sticky="NEWS")
        self.value_combobox.grid(
            row=1, column=1, padx=5, pady=5, sticky="NEWS")
        self.graph_combobox.grid(
            row=2, column=1, padx=5, pady=5, sticky="NEWS")

    def init_button(self):
        """Initialize button"""
        self.plot_button = tk.Button(
            self.frame_panel, text='Plot', command=self.plot_handler)
        self.clr_button = tk.Button(
            self.frame_panel, text='Clear', command=self.clear_handler)
        self.ext_button = tk.Button(
            self.frame_panel, text='Exit', command=self.exit_handler)
        self.save_button = tk.Button(
            self.frame_panel, text='Save', command=self.save_handler, state='disabled'
        )

        self.plot_button.grid(row=0, column=2, padx=5, pady=5, sticky="NEWS")
        self.clr_button.grid(row=1, column=2, padx=5, pady=5, sticky="NEWS")
        self.ext_button.grid(row=2, column=2, padx=5, pady=5, sticky="NEWS")
        self.save_button.grid(row=3, column=2, padx=5, pady=5, sticky="NEWS")

    def init_graph(self):
        """Initialize graph"""
        self.fig = Figure()
        self.axes = self.fig.add_subplot()
        self.fig_canvas = FigureCanvasTkAgg(self.fig, master=self)
        self.fig_canvas.get_tk_widget().grid(
            row=1, column=0, sticky="NEWS", padx=5, pady=5)

    def plot_handler(self):
        get_value1 = self.value_combobox.get()
        get_value2 = self.value2_combobox.get()
        value = [get_value1]
        if self.ck_var.get() == 1 and self.value2_combobox.get() == '':
            return
        if self.ticker_combobox.get() == '' or self.value_combobox.get() == '' or self.graph_combobox.get() == '':
            return
        if get_value1 == get_value2:
            return
        if self.ck_var.get() == 1 and self.value2_combobox.get() != '':
            value.append(get_value2)
        self.axes.clear()
        self.sControl.get_all_data_from_ui(self.ui_data, self.ticker_combobox.get(
        ), value, self.graph_combobox.get(), self.axes)
        self.sControl.plotting()
        self.fig_canvas.draw()
        self.save_button['state'] = 'active'

    def clear_handler(self):
        self.ticker_combobox.set('')
        self.value_combobox.set('')
        self.value2_combobox.set('')
        self.graph_combobox.set('')
        self.axes.clear()
        self.fig_canvas.draw()
        self.save_button['state'] = 'disabled'

    def exit_handler(self):
        self.destroy()

    def save_handler(self):
        file = filedialog.asksaveasfilename(
            title='Save File', filetypes=(("PNG file", "*.png"),))
        self.fig.get_figure().savefig(file)

    def show_progress(self):
        '''Use Toplevel For Show progressbar while loading the data'''
        self.popup = Toplevel(self)
        self.popup.overrideredirect(True)
        self.pg_bttn = ttk.Progressbar(
            self.popup, length=100, mode="determinate")
        self.status = tk.Label(self.popup, text="Stopped")
        self.pg_bttn.grid(row=0, column=0, sticky="NEWS", padx=5, pady=5)
        self.status.grid(row=1, column=0, sticky="NEWS", padx=5, pady=5)
        self.run_task()
        self.tkraise()

    def run_task(self):
        '''start task and thread'''
        self.task_thread = Thread(target=self.task)
        self.task_thread.start()
        self.status.config(text="Running...")
        self.pg_bttn.start()
        self.after(10, self.check_task)

    def check_task(self):
        '''check thread and task if finish stop the progress bar'''
        if self.task_thread.is_alive():
            self.after(10, self.check_task)
        else:
            self.status.config(text="Done.")
            self.pg_bttn.stop()
            self.load_cbb()
            self.popup.destroy()

    def task(self):
        '''load data'''
        self.ui_data = self.sControl.load_data()

    def load_cbb(self):
        '''Load the values to the combobox'''
        self.ticker_combobox['values'] = self.sControl.ticker_for_cbb()
        self.ticker_combobox['state'] = 'readonly'
        self.value_combobox['values'] = self.sControl.value_for_cbb()
        self.value_combobox['state'] = 'readonly'
        self.graph_combobox['values'] = self.sControl.graphtype_for_cbb()
        self.graph_combobox['state'] = 'readonly'

    def run(self):
        """Run the application"""
        self.mainloop()
